import uuid

from faker import Faker
import requests_mock
from assembly_payments.client import AssemblyClient, TEST_BASE_URL
from assembly_payments.types import Callback
from tests.utils.base import PyAssemblyPaymentsTestCase
from tests.utils.factories import CallbackFactory

fake = Faker()


@requests_mock.Mocker()
class TestCallback(PyAssemblyPaymentsTestCase):
    def test_can_list_callbacks(self, mocker):
        self._setup_auth(mocker)
        mocker.get(f"{TEST_BASE_URL}/callbacks", json={"callbacks": [CallbackFactory()]})

        client = AssemblyClient()
        assert client.callbacks.list() != []

    def test_can_get_webhook(self, mocker):
        self._setup_auth(mocker)

        callback = CallbackFactory()
        mocker.get(f"{TEST_BASE_URL}/callbacks/{callback['id']}", json=callback)

        client = AssemblyClient()
        assert client.callbacks.get(callback['id']) == Callback(**callback)

    def test_can_create_callback(self, mocker):
        self._setup_auth(mocker)
        data = dict(
            url=fake.url(),
            object_type="transactions",
            enabled=True,
            description=fake.word()
        )

        mocker.post(f"{TEST_BASE_URL}/callbacks", json=CallbackFactory(**data))

        client = AssemblyClient()
        assert client.callbacks.create(**data).url == data['url']
        assert mocker.last_request.json() == data
        assert mocker.last_request.url == f"{TEST_BASE_URL}/callbacks"
        assert mocker.last_request.method == "POST"

    def test_can_update_webhook(self, mocker):
        self._setup_auth(mocker)
        callback_id = str(uuid.uuid4())
        data = dict(
            url=fake.url(),
            object_type="transactions",
            enabled=True,
            description=fake.word()
        )

        mocker.patch(f"{TEST_BASE_URL}/callbacks/{callback_id}", json={"callbacks": CallbackFactory(id=callback_id, **data)})

        client = AssemblyClient()
        assert client.callbacks.update(callback_id, **data).id == callback_id
        assert mocker.last_request.json() == data
        assert mocker.last_request.url == f"{TEST_BASE_URL}/callbacks/{callback_id}"
        assert mocker.last_request.method == "PATCH"

    def test_can_delete_webhook(self, mocker):
        self._setup_auth(mocker)

        callback = CallbackFactory()
        mocker.delete(f"{TEST_BASE_URL}/callbacks/{callback['id']}", json={
            "status": "Deleted",
            "message": "Successfully redacted"
        })

        client = AssemblyClient()
        client.callbacks.delete(callback['id'])
        assert mocker.last_request.url == f"{TEST_BASE_URL}/callbacks/{callback['id']}"
        assert mocker.last_request.method == "DELETE"

