from datetime import datetime, timedelta
from urllib.parse import urlencode

from faker import Faker
import requests_mock
from assembly_payments.client import AssemblyClient, TEST_BASE_URL
from assembly_payments.types import User, Item
from tests.utils.base import PyAssemblyPaymentsTestCase
from tests.utils.factories import UserFactory, ItemFactory

fake = Faker()


@requests_mock.Mocker()
class TestItem(PyAssemblyPaymentsTestCase):
    def test_can_list_items(self, mocker):
        self._setup_auth(mocker)
        mocker.get(f"{TEST_BASE_URL}/items", json={"items": [ItemFactory()]})

        client = AssemblyClient()
        assert client.items.list() != []

    def test_can_get_item(self, mocker):
        self._setup_auth(mocker)

        item = ItemFactory()
        mocker.get(f"{TEST_BASE_URL}/items/{item['id']}", json={"items": item})

        client = AssemblyClient()
        assert client.items.get(item["id"]) == Item(**item)

    def test_can_get_item_seller(self, mocker):
        self._setup_auth(mocker)

        item = ItemFactory()
        user = UserFactory()
        mocker.get(f"{TEST_BASE_URL}/items/{item['id']}/sellers", json={"users": user})

        client = AssemblyClient()
        assert client.items.get_seller(item["id"]) == User(**user)

    def test_can_get_item_buyer(self, mocker):
        self._setup_auth(mocker)

        item = ItemFactory()
        user = UserFactory()
        mocker.get(f"{TEST_BASE_URL}/items/{item['id']}/buyers", json={"users": user})

        client = AssemblyClient()
        assert client.items.get_buyer(item["id"]) == User(**user)

    def test_can_create_item(self, mocker):
        self._setup_auth(mocker)
        data = dict(
            id=fake.word(),
            name=fake.word(),
            amount=fake.pyint(),
            payment_type=2,
            buyer_id=UserFactory()["id"],
            seller_id=UserFactory()["id"],
            description=fake.word(),
        )

        mocker.post(f"{TEST_BASE_URL}/items", json={"items": ItemFactory(**data)})

        client = AssemblyClient()
        assert client.items.create(**data).id == data["id"]
        assert mocker.last_request.json() == data
        assert mocker.last_request.url == f"{TEST_BASE_URL}/items"
        assert mocker.last_request.method == "POST"

    def test_can_make_payment(self, mocker):
        self._setup_auth(mocker)

        item = ItemFactory()
        data = dict(account_id=fake.word(),)

        mocker.patch(
            f"{TEST_BASE_URL}/items/{item['id']}/make_payment", json={"items": item}
        )

        client = AssemblyClient()
        assert client.items.make_payment(item["id"], **data).id == item["id"]
        assert mocker.last_request.json() == data
        assert (
            mocker.last_request.url
            == f"{TEST_BASE_URL}/items/{item['id']}/make_payment"
        )
        assert mocker.last_request.method == "PATCH"

    def test_can_cancel_payment(self, mocker):
        self._setup_auth(mocker)

        item = ItemFactory()

        mocker.patch(f"{TEST_BASE_URL}/items/{item['id']}/cancel", json={"items": item})

        client = AssemblyClient()
        assert client.items.cancel_payment(item["id"]).id == item["id"]
        assert mocker.last_request.url == f"{TEST_BASE_URL}/items/{item['id']}/cancel"
        assert mocker.last_request.method == "PATCH"

    def test_can_refund_payment(self, mocker):
        self._setup_auth(mocker)

        item = ItemFactory()
        data = dict()

        mocker.patch(f"{TEST_BASE_URL}/items/{item['id']}/refund", json={"items": item})

        client = AssemblyClient()
        assert client.items.refund_payment(item["id"], **data).id == item["id"]
        assert mocker.last_request.json() == data
        assert mocker.last_request.url == f"{TEST_BASE_URL}/items/{item['id']}/refund"
        assert mocker.last_request.method == "PATCH"

    def test_can_authorize_payment(self, mocker):
        self._setup_auth(mocker)

        item = ItemFactory()
        data = dict(account_id=fake.word())

        mocker.patch(
            f"{TEST_BASE_URL}/items/{item['id']}/authorize_payment",
            json={"items": item},
        )

        client = AssemblyClient()
        assert client.items.authorize_payment(item["id"], **data).id == item["id"]
        assert mocker.last_request.json() == data
        assert (
            mocker.last_request.url
            == f"{TEST_BASE_URL}/items/{item['id']}/authorize_payment"
        )
        assert mocker.last_request.method == "PATCH"

    def test_can_capture_payment(self, mocker):
        self._setup_auth(mocker)

        item = ItemFactory()

        mocker.patch(
            f"{TEST_BASE_URL}/items/{item['id']}/capture_payment", json={"items": item}
        )

        client = AssemblyClient()
        assert client.items.capture_payment(item["id"]).id == item["id"]
        assert (
            mocker.last_request.url
            == f"{TEST_BASE_URL}/items/{item['id']}/capture_payment"
        )
        assert mocker.last_request.method == "PATCH"

    def test_can_void_payment(self, mocker):
        self._setup_auth(mocker)

        item = ItemFactory()

        mocker.patch(
            f"{TEST_BASE_URL}/items/{item['id']}/void_payment", json={"items": item}
        )

        client = AssemblyClient()
        assert client.items.void_payment(item["id"]).id == item["id"]
        assert (
            mocker.last_request.url
            == f"{TEST_BASE_URL}/items/{item['id']}/void_payment"
        )
        assert mocker.last_request.method == "PATCH"

    def test_can_filter_items(self, mocker):
        self._setup_auth(mocker)

        offset = 10
        limit = 100
        search = 'ABC'

        now = datetime.now()
        before = now + timedelta(hours=1)
        after = now - timedelta(hours=1)

        params = dict(
            offset=offset,
            limit=limit,
            search=search,
        )

        mocker.get(f"{TEST_BASE_URL}/items?{urlencode(params)}&created_before={before.isoformat()}&created_after={after.isoformat()}", json={"items": [ItemFactory()]})

        params.update(dict(
            created_before=before,
            created_after=after
        ))

        client = AssemblyClient()
        assert client.items.list(**params) != []
