from urllib.parse import urlencode

from faker import Faker
import requests_mock
from assembly_payments.client import AssemblyClient, TEST_BASE_URL
from assembly_payments.types import User, WalletAccount, BankAccount, UserBase, Item
from tests.utils.base import PyAssemblyPaymentsTestCase
from tests.utils.factories import UserFactory, WalletAccountFactory, BankAccountFactory, ItemFactory

fake = Faker()


@requests_mock.Mocker()
class TestUser(PyAssemblyPaymentsTestCase):
    def test_can_list_users(self, mocker):
        self._setup_auth(mocker)
        mocker.get(f"{TEST_BASE_URL}/users", json={"users": [UserFactory()]})

        client = AssemblyClient()
        assert client.users.list() != []

    def test_can_get_user(self, mocker):
        self._setup_auth(mocker)

        user = UserFactory()
        mocker.get(f"{TEST_BASE_URL}/users/{user['id']}", json={"users": user})

        client = AssemblyClient()
        assert client.users.get(user['id']) == User(**user)

    def test_can_create_user(self, mocker):
        self._setup_auth(mocker)
        data = dict(
            id=fake.word(),
            first_name=fake.word(),
            last_name=fake.word(),
            email=fake.email(),
            country="AUS"
        )

        mocker.post(f"{TEST_BASE_URL}/users", json={"users": UserFactory(**data)})

        client = AssemblyClient()
        assert client.users.create(**data).id == data['id']
        assert mocker.last_request.json() == data
        assert mocker.last_request.url == f"{TEST_BASE_URL}/users"
        assert mocker.last_request.method == "POST"

    def test_can_update_user(self, mocker):
        self._setup_auth(mocker)
        data = UserFactory()
        user = UserBase(**data)

        mocker.patch(f"{TEST_BASE_URL}/users/{data['id']}", json={"users": data})

        client = AssemblyClient()
        assert client.users.update(**data).id == data['id']
        assert mocker.last_request.json() == user.model_dump()
        assert mocker.last_request.url == f"{TEST_BASE_URL}/users/{data['id']}"
        assert mocker.last_request.method == "PATCH"

    def test_can_get_user_wallet_account(self, mocker):
        self._setup_auth(mocker)

        user = UserFactory()
        wallet_account = WalletAccountFactory()
        mocker.get(f"{TEST_BASE_URL}/users/{user['id']}/wallet_accounts", json={"wallet_accounts": wallet_account})

        client = AssemblyClient()
        assert client.users.get_wallet_account(user['id']) == WalletAccount(**wallet_account)

    def test_can_get_user_bank_account(self, mocker):
        self._setup_auth(mocker)

        user = UserFactory()
        bank_account = BankAccountFactory()
        mocker.get(f"{TEST_BASE_URL}/users/{user['id']}/bank_accounts", json={"bank_accounts": bank_account})

        client = AssemblyClient()
        assert client.users.get_bank_account(user['id']) == BankAccount(**bank_account)

    def test_can_set_users_disbursement_account(self, mocker):
        self._setup_auth(mocker)

        user = UserFactory()
        bank_account = BankAccountFactory()
        mocker.patch(f"{TEST_BASE_URL}/users/{user['id']}/disbursement_account", json={"users": User(**user).model_dump()})

        client = AssemblyClient()
        assert client.users.set_disbursement_account(user_id=user['id'], account_id=bank_account["id"]).id == user['id']
        assert mocker.last_request.json() == {"account_id": bank_account['id']}
        assert mocker.last_request.url == f"{TEST_BASE_URL}/users/{user['id']}/disbursement_account"
        assert mocker.last_request.method == "PATCH"

    def test_can_get_user_items(self, mocker):
        self._setup_auth(mocker)

        user = UserFactory()
        item = ItemFactory()
        mocker.get(f"{TEST_BASE_URL}/users/{user['id']}/items", json={"items": [Item(**item).model_dump()]})

        client = AssemblyClient()
        assert client.users.list_user_items(user_id=user['id'])[0].id == item['id']
        assert mocker.last_request.url == f"{TEST_BASE_URL}/users/{user['id']}/items"
        assert mocker.last_request.method == "GET"

    def test_can_get_user_items_with_params(self, mocker):
        self._setup_auth(mocker)

        user = UserFactory()
        item = ItemFactory()

        params = dict(
            offset=10,
            limit=100,
        )

        mocker.get(f"{TEST_BASE_URL}/users/{user['id']}/items?{urlencode(params)}", json={"items": [Item(**item).model_dump()]})

        client = AssemblyClient()
        assert client.users.list_user_items(user_id=user['id'], **params)[0].id == item['id']
        assert mocker.last_request.url == f"{TEST_BASE_URL}/users/{user['id']}/items?{urlencode(params)}"
        assert mocker.last_request.method == "GET"
