from faker import Faker
import requests_mock
from assembly_payments.client import AssemblyClient, TEST_BASE_URL, TEST_BETA_URL
from assembly_payments.types import (
    User,
    WalletAccount,
    NppDetails,
    BpayDetails,
    VirtualAccount,
)
from tests.utils.base import PyAssemblyPaymentsTestCase
from tests.utils.factories import (
    UserFactory,
    WalletAccountFactory,
    NppDetailsFactory,
    BpayDetailsFactory,
    VirtualAccountFactory,
    DisbursementFactory,
)

fake = Faker()


@requests_mock.Mocker()
class TestWalletAccount(PyAssemblyPaymentsTestCase):
    def test_can_get_wallet_account(self, mocker):
        self._setup_auth(mocker)

        wallet_account = WalletAccountFactory()
        mocker.get(
            f"{TEST_BASE_URL}/wallet_accounts/{wallet_account['id']}",
            json={"wallet_accounts": wallet_account},
        )

        client = AssemblyClient()
        assert client.wallet_accounts.get(wallet_account["id"]) == WalletAccount(
            **wallet_account
        )

    def test_can_get_wallet_account_user(self, mocker):
        self._setup_auth(mocker)

        wallet_account = WalletAccountFactory()
        user = UserFactory()
        mocker.get(
            f"{TEST_BASE_URL}/wallet_accounts/{wallet_account['id']}/users",
            json={"users": user},
        )

        client = AssemblyClient()
        assert client.wallet_accounts.get_user(wallet_account["id"]) == User(**user)

    def test_can_show_wallet_account_npp_details(self, mocker):
        self._setup_auth(mocker)

        wallet_account = WalletAccountFactory(npp_details=NppDetailsFactory())
        mocker.get(
            f"{TEST_BASE_URL}/wallet_accounts/{wallet_account['id']}/npp_details",
            json={"wallet_accounts": wallet_account},
        )

        client = AssemblyClient()
        assert client.wallet_accounts.get_wallet_account_npp_details(
            wallet_account["id"]
        ) == NppDetails(**wallet_account["npp_details"])

    def test_can_show_wallet_account_bpay_details(self, mocker):
        self._setup_auth(mocker)

        wallet_account = WalletAccountFactory(bpay_details=BpayDetailsFactory())
        mocker.get(
            f"{TEST_BASE_URL}/wallet_accounts/{wallet_account['id']}/bpay_details",
            json={"wallet_accounts": wallet_account},
        )

        client = AssemblyClient()
        assert client.wallet_accounts.get_wallet_account_bpay_details(
            wallet_account["id"]
        ) == BpayDetails(**wallet_account["bpay_details"])

    def test_can_get_wallet_account_virtual_accounts(self, mocker):
        self._setup_auth(mocker)

        wallet_account = WalletAccountFactory()
        virtual_accounts = [VirtualAccountFactory(), VirtualAccountFactory()]
        mocker.get(
            f"{TEST_BETA_URL}/wallet_accounts/{wallet_account['id']}/virtual_accounts",
            json={"virtual_accounts": virtual_accounts},
        )

        client = AssemblyClient()
        assert client.wallet_accounts.get_virtual_accounts(wallet_account["id"]) == [
            VirtualAccount(**virtual_account) for virtual_account in virtual_accounts
        ]

    def test_can_withdraw_funds(self, mocker):
        self._setup_auth(mocker)

        data = dict(
            account_id=fake.word(), amount=fake.pyint(), custom_descriptor=fake.word()
        )
        disbursement = DisbursementFactory()
        wallet_account_id = fake.word()

        mocker.post(
            f"{TEST_BASE_URL}/wallet_accounts/{wallet_account_id}/withdraw",
            json={"disbursements": disbursement},
        )

        client = AssemblyClient()
        assert (
            client.wallet_accounts.withdraw_funds(wallet_account_id, **data).id
            == disbursement["id"]
        )

        assert mocker.last_request.json() == data
        assert (
            mocker.last_request.url
            == f"{TEST_BASE_URL}/wallet_accounts/{wallet_account_id}/withdraw"
        )
        assert mocker.last_request.method == "POST"

    def test_can_withdraw_funds_with_non_batch_id_in_response(self, mocker):
        self._setup_auth(mocker)

        data = dict(
            account_id=fake.word(), amount=fake.pyint(), custom_descriptor=fake.word()
        )
        disbursement = DisbursementFactory(batch_id=None)
        wallet_account_id = fake.word()

        mocker.post(
            f"{TEST_BASE_URL}/wallet_accounts/{wallet_account_id}/withdraw",
            json={"disbursements": disbursement},
        )

        client = AssemblyClient()
        assert (
            client.wallet_accounts.withdraw_funds(wallet_account_id, **data).id
            == disbursement["id"]
        )

        assert mocker.last_request.json() == data
        assert (
            mocker.last_request.url
            == f"{TEST_BASE_URL}/wallet_accounts/{wallet_account_id}/withdraw"
        )
        assert mocker.last_request.method == "POST"

    def test_can_pay_bill(self, mocker):
        self._setup_auth(mocker)

        data = dict(account_id=fake.word(), amount=fake.pyint())
        disbursement = DisbursementFactory()
        wallet_account_id = fake.word()

        mocker.post(
            f"{TEST_BASE_URL}/wallet_accounts/{wallet_account_id}/bill_payment",
            json={"disbursements": disbursement},
        )

        client = AssemblyClient()
        assert (
            client.wallet_accounts.pay_bill(wallet_account_id, **data).id
            == disbursement["id"]
        )

        assert mocker.last_request.json() == data
        assert (
            mocker.last_request.url
            == f"{TEST_BASE_URL}/wallet_accounts/{wallet_account_id}/bill_payment"
        )
        assert mocker.last_request.method == "POST"
