import uuid

from faker import Faker
import requests_mock
from assembly_payments.client import AssemblyClient, TEST_BETA_URL
from assembly_payments.types import Job, Webhook
from tests.utils.base import PyAssemblyPaymentsTestCase
from tests.utils.factories import WebhookFactory, JobFactory

fake = Faker()


@requests_mock.Mocker()
class TestWebhook(PyAssemblyPaymentsTestCase):
    def test_can_list_webhooks(self, mocker):
        self._setup_auth(mocker)
        mocker.get(f"{TEST_BETA_URL}/webhooks", json={"webhooks": [WebhookFactory()]})

        client = AssemblyClient()
        assert client.webhooks.list() != []

    def test_can_get_webhook(self, mocker):
        self._setup_auth(mocker)

        webhook = WebhookFactory()
        mocker.get(f"{TEST_BETA_URL}/webhooks/{webhook['uuid']}", json=webhook)

        client = AssemblyClient()
        assert client.webhooks.get(webhook['uuid']) == Webhook(**webhook)

    def test_can_create_webhook(self, mocker):
        self._setup_auth(mocker)
        data = dict(
            http_method="post",
            url=fake.url(),
            object_type="transactions",
        )

        mocker.post(f"{TEST_BETA_URL}/webhooks", json=WebhookFactory(**data))

        client = AssemblyClient()
        assert client.webhooks.create(**data).url == data['url']
        assert mocker.last_request.json() == data
        assert mocker.last_request.url == f"{TEST_BETA_URL}/webhooks"
        assert mocker.last_request.method == "POST"

    def test_can_update_webhook(self, mocker):
        self._setup_auth(mocker)
        webhook_id = str(uuid.uuid4())
        data = dict(
            http_method="post",
            url=fake.url(),
        )

        mocker.patch(f"{TEST_BETA_URL}/webhooks/{webhook_id}", json={"webhooks": WebhookFactory(uuid=webhook_id, **data)})

        client = AssemblyClient()
        assert client.webhooks.update(webhook_id, **data).uuid == webhook_id
        assert mocker.last_request.json() == data
        assert mocker.last_request.url == f"{TEST_BETA_URL}/webhooks/{webhook_id}"
        assert mocker.last_request.method == "PATCH"

    def test_can_delete_webhook(self, mocker):
        self._setup_auth(mocker)

        webhook = WebhookFactory()
        mocker.delete(f"{TEST_BETA_URL}/webhooks/{webhook['uuid']}", json={
            "status": "Deleted",
            "message": "Successfully redacted"
        })

        client = AssemblyClient()
        client.webhooks.delete(webhook['uuid'])
        assert mocker.last_request.url == f"{TEST_BETA_URL}/webhooks/{webhook['uuid']}"
        assert mocker.last_request.method == "DELETE"

    def test_can_list_webhook_jobs(self, mocker):
        self._setup_auth(mocker)

        job = JobFactory()
        mocker.get(f"{TEST_BETA_URL}/webhooks/{job['webhook_uuid']}/jobs", json={"jobs": [job]})

        client = AssemblyClient()
        assert client.webhooks.list_jobs(job['webhook_uuid']) == [Job(**job)]
